const state = {
    loadingPaymentTerms: false,
    paymentTerms: [],
    paymentMethods: ['Efectivo', 'Banco'],
    paymentTerm: null,
    paymentType: 'cash',
    paymentMethod: 'Efectivo',
    initialPayment: 0,
    paymentResidual: 0,
    paymentLines: []
}

const getters = {
    paymentTerms(state) {
        return state.paymentTerms
    },
    paymentMethods(state) {
        return state.paymentMethods.filter(p => p !== state.paymentMethod)
    },
    loadingPaymentTerms(state) {
        return state.loadingPaymentTerms
    },
    paymentTerm(state) {
        return state.paymentTerm
    },
    paymentType(state) {
        return state.paymentType
    },
    paymentMethod(state) {
        return state.paymentMethod
    },
    initialPayment(state) {
        return state.initialPayment
    },
    paymentLines(state) {
        return state.paymentLines
    }
}

const mutations = {
    setPaymentTerms(state, payload) {
        state.paymentTerms = payload
    },
    setLoadingPaymentTerms(state, payload) {
        state.loadingPaymentTerms = !!payload
    },
    autoSelectPaymentTerm(state, payload) {
        if (!payload || payload === 'cash') {
            state.paymentTerm = state.paymentTerms.find(t => t.lines.length === 1 && t.lines[0].days === 0) || state.paymentTerms.find(t => t.lines.length === 1 && t.lines[0].days >= 0)
        } else {
            state.paymentTerm = state.paymentTerms.find(t => t.lines[0].days > 0)
        }
    },
    setPaymentTerm(state, payload) {
        if (!payload) {
            state.paymentTerm = payload
            return
        }

        state.paymentTerm = state.paymentTerms.find(item => item.id === payload)
    },
    setPaymentType(state, payload) {
        state.paymentType = payload
    },
    setPaymentMethod(state, payload) {
        state.paymentMethod = payload
    },
    setInitialPayment(state, payload) {
        state.initialPayment = payload
    },
    setPaymentLines(state, payload) {
        state.paymentLines = []

        if (!payload) {
            return
        }

        let percentPaid = state.initialPayment / payload.total
        let distributedPercentage = -(percentPaid / state.paymentTerm.lines.length)
        let totals = []
        let residual = payload.total
        let dueDate = null

        for (let line of state.paymentTerm.lines) {
            dueDate = moment(payload.date).add(line.days + line.days2, 'days').format('YYYY-MM-DD')

            if(percentPaid && percentPaid < 1) {
                totals.push([payload.date, percentPaid])
                percentPaid = 0

                if (dueDate === payload.date) {
                    distributedPercentage = ((totals[0][1] - line.valueAmount) / (state.paymentTerm.lines.length - 1))
                    continue
                }
            }

            if (line.value !== 'balance') {
                totals.push([dueDate, line.valueAmount + distributedPercentage])
                continue
            }

            totals.push([dueDate, line.valueAmount])

            for (let line of totals) {
                let currentPrice = (payload.total * line[1]).toFixed(2)

                if (currentPrice < 0) {
                    continue
                }

                residual = residual - currentPrice

                state.paymentLines.push({
                    date: line[0],
                    total: currentPrice !== parseFloat(0).toFixed(2) ? currentPrice : residual.toFixed(2)
                })
            }

            totals = []
        }
    }
}

const actions = {
    initPaymentTerms({ commit }, payload) {
        commit('setPaymentTerms', payload)
        commit('autoSelectPaymentTerm')
        commit('setLoadingPaymentTerms')
    },
    selectPaymentTerm({ commit }, payload) {
        if (!payload) {
            return
        }

        commit('setPaymentTerm', payload)
    },
    changePaymentType({ commit }, payload) {
        commit('setPaymentType', payload)
        commit('autoSelectPaymentTerm', payload)
        commit('setInitialPayment', 0)
        commit('setPaymentLines')
    },
    changePaymentMethod({ commit, dispatch }, methodName) {
        if (methodName === 'Banco') {
            dispatch('toggleBankPayment')
        }

        commit('setPaymentMethod', methodName)
    },
    changeInitialPayment({ commit }, payload) {
        commit('setInitialPayment', payload)
    },
    computePaymentLines({ commit, getters }) {
        commit('setPaymentLines', {
            date: getters.date,
            total: getters.cartTotal
        })
    },
    resetPayment({ commit }) {
        commit('setLoading', true)
        commit('setPaymentTerms', [])
        commit('setPaymentTerm', null)
        commit('setPaymentType', 'cash')
        commit('setInitialPayment', 0)
    }
}

export default {
    state,
    getters,
    mutations,
    actions
}